<?php
/*
 * last modified---
 *		06-30-11/kbw/ new
 *
 *	purpose---
 *		process an incoming voucher payment
 *
 *	inputs---
 *		in $_POST:
 *			pmtKey	-	the baggage field name to look for
 *			keyVal	-	the baggage field value identifying the incoming payment
 *		(plus anything else which should be passed to the success/failure URLs)
 */
// open log file
openlog("VS Web SCI", LOG_PID, LOG_USER);

// the name of the JAR file which contains the Voucher-Safe CLI executable
$execJAR = "/data/www/vouchers/websci/java/voucher-cli.jar";

// the OFS gateway hostname
$OFShost = "ofs.vm.to";

// the OFS gateway port number
$OFSport = 5233;

// the location to which we direct the user on successful payment pickup
$successURL = "pmtSuccess.php";

// the location to which we direct the user on payment pickup failure
$failureURL = "pmtFailure.php";

// NB: the login details for the payment receiving safe must be stored
// 	 separately in the file ~/.vouchers/.vshlogin (where ~ is the home
// 	 directory of the user running the web server, e.g. www-data).

// invoke the CLI, telling it what to look for
// Notes: 'java' must be in the path.  Also, if you have spaces in either the
// 		$pmtKey or $keyVal, you'll need to add enclosing double quotes.

$pmtKey = $_POST['pmtKey'];
$keyVal = $_POST['keyVal'];
$invoke = "java -jar $execJAR -x $OFShost -p $OFSport -w -b $pmtKey:$keyVal";

// invoke the CLI client
$result = exec($invoke);

/* The value of $result will be one of these:
 *	1. "NOT FOUND" - if the payment could not be found
 *	2. a JSON string - if the payment was received successfully
 *	3. an error - otherwise
 */
$redirArgs = array();
$redirArgs[$pmtKey] = $keyVal;
if ($result === "NOT FOUND") {
	syslog(LOG_ERR, "Payment not found for key $pmtKey:$keyVal");
	http_redirect($failureURL, $redirArgs);
}
else {
	$pmtDesc = json_decode($result, true);
	if (is_null($pmtDesc) || !is_array($pmtDesc) || empty($pmtDesc)) {
		// some other kind of error
		syslog(LOG_ERR,
				"Error receiving payment for key $pmtKey:$keyVal : $result");
		http_redirect($failureURL, $redirArgs);
	}
	else {
		// Certain items are found in the JSON, which should be recorded or
		// confirmed to match the order.
		
		$baggage = array();
		foreach ($pmtDesc as $field => $value) {
			$redirArgs[$field] = $value;
			switch ($field) {
			case 'PAYER':
				// the customer's VS#
				$payerVS = $value;
				break;
			case 'AMOUNT':
				// echo of amount (should be confirmed)
				$amount = $value;
				break;
			case 'UNITS':
				// echo of amount units (should be confirmed)
				$units = $value;
				break;
			case 'ASSET':
				// voucher asset type
				$asset = $value;
				break;
			case 'ISSUER':
				// echo of asset Issuer (should be confirmed)
				$issuer = $value;
				break;
			default:
				// baggage field
				if ($field != $pmtKey) {
					$baggage[$field] = $value;
				}
				break;
			}
		}
		syslog(LOG_NOTICE, "Payment received: from $payerVS, $amount $units of "
				. "$asset at $issuer, $pmtKey = $keyVal, baggage: "
				. print_r($baggage, true));
		http_redirect($successURL, $redirArgs);
	}
}
closelog();
?>
